/*  Sashi89 - TI89-Amiga communication program
 *  Copyright (C) 2000, 2001  Olivier Croquette (ocroquette@free.fr)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

/* Remote control of the TI */

/* We just open a window with a custom gadget :
	 an invisible drawing area, which just catch the keystrokes
	 and send them to the TI
*/

/*
#define DEBUG
*/

/* ANSI : */
#include <stdio.h>
#include <ctype.h>


/* Amiga : */
#include <proto/intuition.h>
#include <proto/exec.h>
#include <clib/muimaster_protos.h>
#include <libraries/mui.h>
#include <devices/inputevent.h>


#include "mui_remote.h"
#include "protocol.h"
#include "ti89_keys.h"




#define MUIV_Remote_Cancel (TAG_USER|(666<<16)|0x1001)
#define MUIV_Remote_OK (TAG_USER|(666<<16)|0x1001)


ULONG mui_remote_cleanup(struct IClass *cl,Object *obj,struct MUIP_HandleInput *msg);
ULONG mui_remote_setup(struct IClass *cl,Object *obj,struct MUIP_HandleInput *msg);
ULONG mui_remote_handleinput(struct IClass *cl,Object *obj,struct MUIP_HandleInput *msg);
ULONG mui_remote_dispatcher(struct IClass *cl __asm("a0"),Object *obj __asm("a2"),Msg msg __asm("a1"));


typedef struct
{ /* Just for the day we will need data attached to the custom gadget */
	int dumy;
} mui_remote_data;




ULONG mui_remote_setup(struct IClass *cl,Object *obj,struct MUIP_HandleInput *msg)
{
/*	mui_remote_data	*data = INST_DATA(cl,obj);*/

  if (!(DoSuperMethodA(cl,obj,(Msg)msg)))
    return(FALSE);

	MUI_RequestIDCMP(obj,IDCMP_VANILLAKEY|IDCMP_RAWKEY);

 return(TRUE);
}


ULONG mui_remote_cleanup(struct IClass *cl,Object *obj,struct MUIP_HandleInput *msg)
{
 MUI_RejectIDCMP(obj,IDCMP_VANILLAKEY|IDCMP_RAWKEY);
 return(DoSuperMethodA(cl,obj,(Msg)msg));
}


ULONG mui_remote_handleinput(struct IClass *cl,Object *obj,struct MUIP_HandleInput *msg)
{
	int shift_pressed;
	int lamiga_pressed;
	int ramiga_pressed;

	/* This is the code we will send to the TI : */
	unsigned short int ti_code;
	UWORD key_code;

	ti_code  = 0x0000;
	key_code = msg->imsg->Code;

#ifdef DEBUG
	printf("class=0x%-8x key_code=0x%-8x  qualifier=0x%-8x\n",msg->imsg->Class,key_code,msg->imsg->Qualifier);
#endif

	shift_pressed  = msg->imsg->Qualifier & ( IEQUALIFIER_LSHIFT | IEQUALIFIER_RSHIFT);
	lamiga_pressed = msg->imsg->Qualifier & ( IEQUALIFIER_LCOMMAND );
	ramiga_pressed = msg->imsg->Qualifier & ( IEQUALIFIER_RCOMMAND );

	if (  msg->imsg->Class == IDCMP_VANILLAKEY )
  {
		switch(key_code)
		{

			case 0x08: /* Backspace */
				if ( shift_pressed )
				{ /* Simulate CLEAR with SHIFT+DEL */
					ti_code = TI89_KEY_CLEAR;
					shift_pressed = 0;
				}
				else
					ti_code = TI89_KEY_BS;

				break;
			case 0x1b: /* ESC */
				ti_code = TI89_KEY_ESC;
				break;
			default:
				ti_code = key_code;
				break;
		}
  }
	if (  msg->imsg->Class == IDCMP_RAWKEY )
  {
		switch(key_code)
		{			 
				/* Cursor keys */

			case 0x4c: /* UP */
				ti_code = TI89_KEY_UP;
				break;
			case 0x4d: /* DOWN */
				ti_code = TI89_KEY_DOWN;
				break;
			case 0x4f: /* RIGHT */
				ti_code = TI89_KEY_RIGHT;
				break;
			case 0x4e: /* LEFT */
				ti_code = TI89_KEY_LEFT;
				break;


			case 0x58: /* F9 */
				ti_code = TI89_KEY_APPS;
				break;

			case 0x66: /* F9 */
				ti_code = TI89_KEY_CTRL << 8;
				break;
			
			default:
				if ( key_code >= 0x50 && key_code <= 0x57 ) /* F keys */
				{
					ti_code = TI89_KEY_F1 + key_code - 0x50;
					break;
				}
		}
	}
#ifdef DEBUG
	printf("ti_code = %x\n",ti_code);
#endif

	if ( ti_code != 0x0000 )
	{
		if ( lamiga_pressed )
			ti_code |= (TI89_KEY_CTRL<<8);

	
#ifdef DEBUG
		printf("  sending %x ... ",ti_code);
		printf("%d\n",ti_send_key(ti_code));
#endif
	}


  return(DoSuperMethodA(cl,obj,(Msg)msg));
}


ULONG mui_remote_dispatcher(struct IClass *cl __asm("a0"),Object *obj __asm("a2"),Msg msg __asm("a1"))
{
  switch (msg->MethodID)
  {
		case MUIM_HandleInput: return(mui_remote_handleinput(cl,obj,(APTR)msg));
		case MUIM_Setup  : return(mui_remote_setup  (cl,obj,(APTR)msg));
		case MUIM_Cleanup : return(mui_remote_cleanup (cl,obj,(APTR)msg));
  }

  return(DoSuperMethodA(cl,obj,(Msg)msg));
}



int mui_remote(Object *app)
{
	Object *win=NULL;
	Object *bt_quit=NULL;
	Object *area=NULL;
	struct MUI_CustomClass *mcc=NULL;
	int ok=0;

	if (!(mcc = MUI_CreateCustomClass(NULL,MUIC_Area,NULL,sizeof(mui_remote_data),mui_remote_dispatcher)))
	{
		fprintf(stderr,"Could not create custom class.\n");
		goto mui_remote_cleanup;
	}

	win= WindowObject,
		MUIA_Window_Title,"TI-Remote",
    WindowContents,
      VGroup,
				Child,
					area = NewObject(mcc->mcc_Class,NULL,TAG_DONE),
        Child,ColGroup(2),
					Child, KeyLabel2("Remote control in progress",'f'),
        End,
        Child,
          HGroup,
						Child, bt_quit=SimpleButton("Quit"),
          End,
      End,
	End;

  if ( win == NULL )
  {
    printf("Unable to open window.\n");
		return(0);
  }


  set(app,MUIA_Application_Sleep,TRUE);
	set(bt_quit,MUIA_CycleChain,TRUE);


	DoMethod(bt_quit,MUIM_Notify,MUIA_Pressed,FALSE,
            app,2,MUIM_Application_ReturnID,MUIV_Remote_OK);

  DoMethod(win,MUIM_Notify,MUIA_Window_CloseRequest,TRUE,
           app,2,MUIM_Application_ReturnID,MUIV_Remote_OK);


  DoMethod(app,OM_ADDMEMBER,win);

  set(win,MUIA_Window_Open,TRUE);

  {
    int sigs;
		unsigned long int resultwin;

    while ( (resultwin=DoMethod(app,MUIM_Application_Input,&sigs)) != MUIV_Remote_Cancel
          && resultwin!=MUIV_Remote_OK )
    {
      if ( sigs )
        Wait(sigs);
    }
  }

  set(win,MUIA_Window_Open,FALSE);

  DoMethod(app,OM_REMMEMBER,win);

	ok = 1;

	/* code to cleanup. we arrive here :
	- if the user clicks quit
	- if there has been a problem with the creation of our window
		with a goto
	*/

	mui_remote_cleanup :

	if ( win )
		MUI_DisposeObject(win);
	
	if ( mcc )
		MUI_DeleteCustomClass(mcc);

  set(app,MUIA_Application_Sleep,FALSE);

	if ( ok )
		return(0);
	else
		return(1);
}

